#include <TFT_eSPI.h>
#include <CST816S.h>
#include "Free_Fonts.h"
#include <Wire.h>

TFT_eSPI tft = TFT_eSPI();
CST816S touch(5, 6, 10, 7);

String keys[12] = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "*", "0", "#"};
String extraKeys[3] = {"Call", "Back", "Delete"};
String text = "";
String contacts[3][2] = {
  {"Ashwini", "123456789"},
  {"Axy", "67876876868"},
  {"Random", "9876543210"}
};

enum Screen { HOME, CONTACTS, CALCULATOR, GAMES, TIMER };
Screen currentScreen = HOME;

void setup() {
  Serial.begin(115200);

  pinMode(TFT_BL, OUTPUT);
  digitalWrite(TFT_BL, HIGH);

  tft.begin();
  tft.fillScreen(TFT_BLACK);
  tft.setTextColor(TFT_WHITE, TFT_BLACK);
  tft.setFreeFont(FSB9);

  tft.setCursor(30, 50); // Adjusted for smaller display
  tft.print(text + "_");
  touch.begin();

  drawHomeScreen();
}

void drawHomeScreen() {
  tft.fillScreen(TFT_BLACK);
  tft.setTextColor(TFT_WHITE, TFT_BLACK);
  
  int buttonWidth = 80, buttonHeight = 60;
  String apps[4] = {"Contacts", "Calculator", "Games", "Timer"};

  // Display buttons for different apps in a grid
  for (int i = 0; i < 4; i++) {
    int x = (i % 2) * (buttonWidth + 10) + 10;
    int y = (i / 2) * (buttonHeight + 10) + 30;
    
    tft.drawRoundRect(x, y, buttonWidth, buttonHeight, 5, TFT_WHITE);
    tft.setCursor(x + 10, y + 20);
    tft.print(apps[i]);
  }
}

void drawCalculator() {
  tft.fillScreen(TFT_BLACK);
  
  String calcKeys[16] = {"1", "2", "3", "+", "4", "5", "6", "-", "7", "8", "9", "*", "C", "0", "=", "/"};
  
  int keyWidth = 35, keyHeight = 35;
  int col = 0, row = 0;
  
  // Display calculator keys
  for (int i = 0; i < 16; i++) {
    int x = col * (keyWidth + 5) + 10;
    int y = row * (keyHeight + 5) + 80;  // Lowered the button position
    
    tft.drawRoundRect(x, y, keyWidth, keyHeight, 5, TFT_WHITE);
    tft.setCursor(x + 20, y + 10);
    tft.print(calcKeys[i]);
    
    col++;
    if (col == 4) {
      col = 0;
      row++;
    }
  }
  
  // Draw result display area
  tft.fillRect(10, 30, 240, 40, TFT_GREEN);  // Lowered result display
  tft.setTextColor(TFT_BLACK, TFT_GREEN);
  tft.setCursor(15, 50); // Lowered text
  tft.print(text + "_");
}

void drawContacts() {
  tft.fillScreen(TFT_BLACK);

  // Display each contact with a separator line
  for (int i = 0; i < 3; i++) {
    tft.setCursor(20, 40 + i * 40);
    tft.setTextColor(TFT_WHITE, TFT_BLACK);
    tft.print(contacts[i][0]);
    
    tft.setCursor(20, 60 + i * 40);
    tft.print(contacts[i][1]);
    
    tft.drawLine(0, 80 + i * 40, 240, 80 + i * 40, TFT_WHITE);  // Draw line separator
  }
}

void loop() {
  if (touch.available()) {
    int x = touch.data.x;
    int y = touch.data.y;

    // Handle touch based on the current screen
    if (currentScreen == HOME) {
      handleHomeScreenTouch(x, y);
    } else if (currentScreen == CALCULATOR) {
      handleCalculatorTouch(x, y);
    } else if (currentScreen == CONTACTS) {
      handleContactTouch(x, y);
    }
  }
}

void handleHomeScreenTouch(int x, int y) {
  if (x > 10 && x < 90 && y > 30 && y < 90) {
    // Contacts button touched
    currentScreen = CONTACTS;
    drawContacts();
  } else if (x > 110 && x < 190 && y > 30 && y < 90) {
    // Calculator button touched
    currentScreen = CALCULATOR;
    drawCalculator();
  }
  // Add handling for other buttons like "Games" and "Timer"
}

void handleCalculatorTouch(int x, int y) {
  int keyWidth = 35, keyHeight = 35;

  // Check which calculator button is touched
  int col = (x - 10) / (keyWidth + 5);
  int row = (y - 80) / (keyHeight + 5);

  if (col >= 0 && col < 4 && row >= 0 && row < 4) {
    int index = row * 4 + col;
    String calcKeys[16] = {"1", "2", "3", "+", "4", "5", "6", "-", "7", "8", "9", "*", "C", "0", "=", "/"};
    
    String keyPressed = calcKeys[index];
    
    if (keyPressed == "=") {
      // Calculate result
      text = calculateResult(text);
    } else if (keyPressed == "C") {
      text = "";
    } else {
      text += keyPressed;
    }

    // Update the result display area
    tft.fillRect(10, 30, 240, 40, TFT_GREEN);
    tft.setCursor(15, 50);
    tft.print(text + "_");
  }
}

void handleContactTouch(int x, int y) {
  // Add code to handle contact touch interactions
  // For example, selecting a contact for more details or making a call
}

String calculateResult(String expression) {
  // Here you can add a simple parser or use a library to evaluate the expression
  // For demonstration, we'll use a simple eval-like logic to handle basic operations
  float result = 0.0;
  char lastOp = '+';
  String currentNum = "";

  for (char c : expression) {
    if (isDigit(c)) {
      currentNum += c;  // Build the current number
    } else {
      if (currentNum.length() > 0) {
        float num = currentNum.toFloat();
        if (lastOp == '+') {
          result += num;
        } else if (lastOp == '-') {
          result -= num;
        } else if (lastOp == '*') {
          result *= num;
        } else if (lastOp == '/') {
          if (num != 0) {
            result /= num;
          } else {
            return "Error";  // Handle division by zero
          }
        }
        currentNum = "";  // Reset current number
      }
      lastOp = c;  // Update last operator
    }
  }

  // Handle the last number in the expression
  if (currentNum.length() > 0) {
    float num = currentNum.toFloat();
    if (lastOp == '+') {
      result += num;
    } else if (lastOp == '-') {
      result -= num;
    } else if (lastOp == '*') {
      result *= num;
    } else if (lastOp == '/') {
      if (num != 0) {
        result /= num;
      } else {
        return "Error";  // Handle division by zero
      }
    }
  }

  // Convert result to string
  return String(result);
}
