#include <TFT_eSPI.h>
#include <CST816S.h>
#include "Free_Fonts.h"
#include <Wire.h>

TFT_eSPI tft = TFT_eSPI();
CST816S touch(5, 6, 10, 7);

String keys[12] = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "*", "0", "#"};
String extraKeys[3] = {"Call", "Back", "Delete"};
String text = "";

void setup() {
  Serial.begin(115200);

  pinMode(TFT_BL, OUTPUT);
  digitalWrite(TFT_BL, HIGH);

  tft.begin();
  tft.fillScreen(TFT_BLACK);
  tft.setTextColor(TFT_WHITE, TFT_BLACK);
  tft.setFreeFont(FSB9);

  tft.setCursor(30, 50); // Adjusted for smaller display
  tft.print(text + "_");
  touch.begin();

  drawKeypad();
}

void drawKeypad() {
  int keyWidth = 30;  // Numeric key width
  int keyHeight = 30; // Numeric key height
  int startX = 35;    // Centered for circular display
  int startY = 60;    // Adjusted Y position to fit the screen

  int col = 0;
  int row = 0;

  // Draw numeric keys
  for (int i = 0; i < 12; i++) {
    int x = startX + (col * (keyWidth + 5));
    int y = startY + (row * (keyHeight + 5));

    tft.drawRoundRect(x, y, keyWidth, keyHeight, 5, TFT_WHITE);
    tft.setCursor(x + 10, y + 10); // Adjusted text position for smaller buttons
    tft.print(keys[i]);

    col++;
    if (col == 3) {
      col = 0;
      row++;
    }
  }

  // Draw extra keys: Call, Back, and Delete
  int extraKeyWidth = 40;  // Set width to 40 pixels
  int extraKeyHeight = 19; // Set height to 19 pixels
  int extraStartX = 30;    // Move the starting X position to the right

  // Adjust the Y position to reduce the gap between numeric keys and extra buttons
  int extraStartY = startY + (row * (keyHeight + 5)) + 5;

  for (int i = 0; i < 3; i++) {
    int x = extraStartX + (i * (extraKeyWidth + 10));
    int y = extraStartY;

    // Set different colors for each button
    if (i == 0) {
      tft.fillRoundRect(x, y, extraKeyWidth, extraKeyHeight, 5, TFT_GREEN);  // Call button
    } else if (i == 1) {
      tft.fillRoundRect(x, y, extraKeyWidth, extraKeyHeight, 5, TFT_LIGHTGREY);  // Back button
    } else {
      tft.fillRoundRect(x, y, extraKeyWidth, extraKeyHeight, 5, TFT_RED);   // Delete button
    }

    tft.setTextColor(TFT_BLACK, TFT_WHITE);
    tft.setCursor(x + 10, y + 5);  // Adjusted text position for smaller buttons
    tft.print(extraKeys[i]);
  }

  tft.setTextColor(TFT_WHITE, TFT_BLACK);  // Reset text color
}

void loop() {
  if (touch.available()) {
    Serial.print(touch.data.points);
    int x = touch.data.x;
    int y = touch.data.y;
    String keyPressed = getKey(x, y);
    if (keyPressed != "") {
      text += keyPressed;
    }

    // Update only the text area, avoid clearing the entire screen
    tft.fillRect(0, 30, 240, 40, TFT_BLACK);  // Clear the text area only
    tft.setCursor(30, 50);  // Adjusted for smaller display
    tft.print(text + "_");
  }
}

String getKey(int x, int y) {
  int keyWidth = 30;  // Adjusted for smaller buttons
  int keyHeight = 30;
  int startX = 35;    // Adjusted for smaller layout
  int startY = 60;

  // Check if within numeric keypad bounds
  if (y >= startY && y < startY + 4 * (keyHeight + 5)) {
    int col = (x - startX) / (keyWidth + 5);
    int row = (y - startY) / (keyHeight + 5);

    int index = row * 3 + col;
    if (index >= 0 && index < 12) {
      return keys[index];
    }
  }

  // Check for extra buttons
  int extraKeyWidth = 40;  // Set width to 40 pixels
  int extraKeyHeight = 19; // Set height to 19 pixels
  int extraStartX = 30;    // Moved slightly to the right
  int extraStartY = startY + 4 * (keyHeight + 10) + 10;

  if (y >= extraStartY && y < extraStartY + extraKeyHeight) {
    int extraIndex = (x - extraStartX) / (extraKeyWidth + 10);
    if (extraIndex >= 0 && extraIndex < 3) {
      if (extraKeys[extraIndex] == "Call") {
        Serial.println("Calling...");
      } else if (extraKeys[extraIndex] == "Back") {
        if (text.length() > 0) {
          text.remove(text.length() - 1);  // Remove last character
        }
      } else if (extraKeys[extraIndex] == "Delete") {
        text = "";  // Clear the text
      }
      return "";
    }
  }

  return "";
}
