#include <TFT_eSPI.h>
#include <CST816S.h>
#include "Free_Fonts.h"
#include <Wire.h>

TFT_eSPI tft = TFT_eSPI();
CST816S touch(5, 6, 10, 7);

#define BLACK TFT_BLACK
#define WHITE TFT_WHITE
#define GREEN TFT_GREEN
#define RED TFT_RED

int16_t h = 160;
int16_t w = 128;

int16_t paddleHeight = 30;
int16_t paddleWidth = 4;
int16_t ballSize = 4;

int16_t leftPaddleY = (h - paddleHeight) / 2;
int16_t rightPaddleY = (h - paddleHeight) / 2;

int16_t ballX = w / 2;
int16_t ballY = h / 2;
int16_t ballDX = 1;
int16_t ballDY = 1;

void setup() {
  Serial.begin(115200);
  
  pinMode(TFT_BL, OUTPUT);
  digitalWrite(TFT_BL, HIGH);

  tft.begin();
  tft.fillScreen(BLACK);
  tft.setTextColor(WHITE, BLACK);
  tft.setFreeFont(FSB9);

  touch.begin();
  drawGame();
}

void loop() {
  if (touch.available()) {
    int x = touch.data.x;
    int y = touch.data.y;
    
    // Move the left paddle
    if (isLeftPaddleTouched(y)) {
      leftPaddleY = constrain(y - paddleHeight / 2, 0, h - paddleHeight);
    }

    // Move the right paddle
    if (isRightPaddleTouched(y)) {
      rightPaddleY = constrain(y - paddleHeight / 2, 0, h - paddleHeight);
    }
  }

  updateBall();
  delay(20);
}

void drawGame() {
  tft.fillScreen(BLACK);
  drawPaddles();
  drawBall();
}

void drawPaddles() {
  tft.fillRect(0, leftPaddleY, paddleWidth, paddleHeight, GREEN);  // Left Paddle
  tft.fillRect(w - paddleWidth, rightPaddleY, paddleWidth, paddleHeight, RED); // Right Paddle
}

void drawBall() {
  tft.fillCircle(ballX, ballY, ballSize, WHITE);
}

void updateBall() {
  // Clear the old ball position
  tft.fillCircle(ballX, ballY, ballSize, BLACK);

  // Update ball position
  ballX += ballDX;
  ballY += ballDY;

  // Ball collision with top and bottom walls
  if (ballY <= ballSize || ballY >= h - ballSize) {
    ballDY *= -1;  // Reverse ball direction on Y-axis
  }

  // Ball collision with paddles
  if (ballX <= paddleWidth && ballY >= leftPaddleY && ballY <= leftPaddleY + paddleHeight) {
    ballDX *= -1;  // Reverse ball direction on X-axis
  } else if (ballX >= w - paddleWidth - ballSize && ballY >= rightPaddleY && ballY <= rightPaddleY + paddleHeight) {
    ballDX *= -1;  // Reverse ball direction on X-axis
  }

  // Reset ball if it goes out of bounds
  if (ballX < 0 || ballX > w) {
    ballX = w / 2;
    ballY = h / 2;
    ballDX = (random(0, 2) * 2 - 1); // Randomize direction
    ballDY = (random(0, 2) * 2 - 1); // Randomize direction
  }

  drawBall(); // Draw new ball position
}

bool isLeftPaddleTouched(int y) {
  return (y >= leftPaddleY && y <= leftPaddleY + paddleHeight);
}

bool isRightPaddleTouched(int y) {
  return (y >= rightPaddleY && y <= rightPaddleY + paddleHeight);
}
